import os
import threading
import sounddevice as sd
import librosa

import numpy as np
import torch
from transformers.models.whisper.modeling_whisper import (
    WhisperForConditionalGeneration,
)
from transformers.models.whisper.processing_whisper import WhisperProcessor
from transformers.models.whisper.feature_extraction_whisper import (
    WhisperFeatureExtractor,
)

from airunner.handlers.base_model_manager import BaseModelManager
from airunner.enums import SignalCode, ModelType, ModelStatus
from airunner.exceptions import NaNException
from airunner.utils.memory import clear_memory
from airunner.settings import (
    AIRUNNER_DEFAULT_STT_HF_PATH,
    AIRUNNER_LOCAL_FILES_ONLY,
)


class WhisperModelManager(BaseModelManager):
    """
    Handler for the Whisper model from OpenAI.
    """

    def __init__(self, *args, **kwargs):
        self.model_type = ModelType.STT
        self.model_class = "stt"
        super().__init__(*args, **kwargs)
        self._lock = threading.Lock()
        self._model = None
        self._processor = None
        self._feature_extractor = None
        self._sampling_rate = 16000
        self.audio_stream = None

    @property
    def dtype(self):
        return torch.bfloat16

    @property
    def stt_is_loading(self):
        return self.model_status is ModelStatus.LOADING

    @property
    def stt_is_loaded(self):
        return self.model_status is ModelStatus.LOADED

    @property
    def stt_is_unloaded(self):
        return self.model_status is ModelStatus.UNLOADED

    @property
    def model_path(self) -> str:
        file_path = os.path.expanduser(
            os.path.join(
                self.path_settings.stt_model_path, AIRUNNER_DEFAULT_STT_HF_PATH
            )
        )
        return os.path.abspath(file_path)

    def process_audio(self, audio_data):
        """
        Process incoming audio data.
        """
        with self._lock:
            item = audio_data["item"]

            # Convert the byte string to a float32 array
            inputs = np.frombuffer(item, dtype=np.int16)
            inputs = inputs.astype(np.float32) / 32767.0

            transcription = None
            try:
                transcription = self._process_inputs(inputs)
            except Exception as e:
                self.logger.error(f"Failed to process inputs {e}")
                self.logger.error(e)

            print("TRANSCRIPTION", transcription)
            if transcription:
                self._send_transcription(transcription)

    def load(self, retry: bool = False):
        if self.stt_is_loading or self.stt_is_loaded:
            return
        self.logger.debug("Loading Whisper (text-to-speech)")
        self.unload()
        self.change_model_status(ModelType.STT, ModelStatus.LOADING)
        self._load_model()
        # unsure why this is failing to load occasionally - this is a hack
        if self._model is None and retry is False:
            return self.load(retry=True)
        self._load_processor()
        self._load_feature_extractor()
        if (
            self._model is not None
            and self._processor is not None
            and self._feature_extractor is not None
        ):
            self.change_model_status(ModelType.STT, ModelStatus.LOADED)
            return True
        else:
            self.change_model_status(ModelType.STT, ModelStatus.FAILED)
            return False

    def unload(self):
        if self.stt_is_loading or self.stt_is_unloaded:
            return
        self.logger.debug("Unloading Whisper (text-to-speech)")
        self.change_model_status(ModelType.STT, ModelStatus.LOADING)
        self._unload_model()
        self._unload_processor()
        self._unload_feature_extractor()
        self.change_model_status(ModelType.STT, ModelStatus.UNLOADED)

    def _load_model(self):
        self.logger.debug(
            f"Loading model from {self.model_path} to device {self.device}"
        )
        device = self.device
        try:
            self._model = WhisperForConditionalGeneration.from_pretrained(
                self.model_path,
                local_files_only=AIRUNNER_LOCAL_FILES_ONLY,
                torch_dtype=self.dtype,
                use_safetensors=True,
                force_download=False,
            )
            self._model.to(device)
        except Exception as e:
            self.logger.error(f"Failed to load model: {e}")
            return None

    def _load_processor(self):
        model_path = self.model_path
        self.logger.debug(f"Loading processor from {model_path}")
        try:
            self._processor = WhisperProcessor.from_pretrained(
                model_path,
                local_files_only=AIRUNNER_LOCAL_FILES_ONLY,
                torch_dtype=self.dtype,
                device_map=self.device,
            )
        except Exception as e:
            self.logger.error(f"Failed to load processor: {e}")
            return None

    def _load_feature_extractor(self):
        model_path = self.model_path
        self.logger.debug(f"Loading feature extractor {model_path}")
        try:
            self._feature_extractor = WhisperFeatureExtractor.from_pretrained(
                model_path,
                local_files_only=AIRUNNER_LOCAL_FILES_ONLY,
                torch_dtype=self.dtype,
                device_map=self.device,
            )
        except Exception as e:
            self.logger.error(f"Failed to load feature extractor")
            self.logger.error(e)
            return None

    def _unload_model(self):
        del self._model
        self._model = None
        clear_memory(self.device)

    def _unload_processor(self):
        del self._processor
        self._processor = None
        clear_memory(self.device)

    def _unload_feature_extractor(self):
        del self._feature_extractor
        self._feature_extractor = None
        clear_memory(self.device)

    def _process_inputs(self, inputs: np.ndarray) -> str:
        if not self._feature_extractor:
            return ""
        inputs = torch.from_numpy(inputs).to(torch.float32).to(self.device)

        if torch.isnan(inputs).any():
            raise NaNException

        # Move inputs to CPU and ensure they are in float32 before
        # passing to _feature_extractor
        inputs = inputs.cpu().to(torch.float32)
        inputs = self._feature_extractor(
            inputs, sampling_rate=self._sampling_rate, return_tensors="pt"
        )

        if torch.isnan(inputs.input_features).any():
            raise NaNException

        inputs["input_features"] = (
            inputs["input_features"].to(self.dtype).to(self.device)
        )
        if torch.isnan(inputs.input_features).any():
            raise NaNException

        transcription = self._run(inputs)
        if transcription is None or "nan" in transcription:
            raise NaNException

        return transcription

    def _run(self, inputs) -> str:
        """
        Run the model on the given inputs.
        :param inputs: str - The transcription of the audio data.
        :return:
        """
        self.logger.debug("Running model")
        try:
            input_tensor = inputs.input_features
        except AttributeError:
            self.logger.error("Input features not found in inputs")
            return ""

        if torch.isnan(input_tensor).any():
            raise NaNException

        # Ensure input tensor is in the correct dtype
        input_tensor = input_tensor.to(self.dtype).to(self.device)

        data = dict(
            input_features=input_tensor,
            is_multilingual=self.whisper_settings.is_multilingual,
            temperature=self.whisper_settings.temperature,
            compression_ratio_threshold=self.whisper_settings.compression_ratio_threshold,
            logprob_threshold=self.whisper_settings.logprob_threshold,
            no_speech_threshold=self.whisper_settings.no_speech_threshold,
            time_precision=self.whisper_settings.time_precision,
        )

        if self.whisper_settings.is_multilingual:
            data["task"] = self.whisper_settings.task

        try:
            generated_ids = self._model.generate(**data)
        except RuntimeError as e:
            self.logger.error(f"Error in model generation: {e}")
            return ""

        if generated_ids is None or torch.isnan(generated_ids).any():
            raise NaNException

        transcription = self.process_transcription(generated_ids)
        if len(transcription) == 0 or len(transcription.split(" ")) == 1:
            return ""

        return transcription

    def _send_transcription(self, transcription: str):
        """
        Emit the transcription so that other handlers can use it
        """
        self.emit_signal(
            SignalCode.AUDIO_PROCESSOR_RESPONSE_SIGNAL,
            {"transcription": transcription},
        )

    def process_transcription(self, generated_ids) -> str:
        # Decode the generated ids
        generated_ids = generated_ids.to("cpu").to(torch.float32)
        transcription = self._processor.batch_decode(
            generated_ids, skip_special_tokens=True
        )[0]

        # Remove leading and trailing whitespace
        transcription = transcription.strip()

        # Remove any extra whitespace
        transcription = " ".join(transcription.split())

        return transcription



import logging
import traceback
import inspect


class Logger:
    def __init__(self, name: str, level: int = logging.DEBUG):
        # Configure the logger
        logger = logging.getLogger(name)
        logger.setLevel(level)

        # Remove all existing handlers
        if logger.hasHandlers():
            logger.handlers.clear()

        handler = logging.StreamHandler()
        handler.setFormatter(logging.Formatter(
            '%(asctime)s - %(name)s - %(levelname)s - %(caller_module)s - %(caller_function)s - %(caller_lineno)d - %(message)s')
        )

        logger.addHandler(handler)

        # Disable propagation to the root logger
        logger.propagate = False
        self.logger = logger
        self.name = name
    
    def _get_caller_info(self):
        """Get the caller module name, function name and line number."""
        frame = inspect.currentframe().f_back.f_back  # Skip this function and the logging method
        module_name = self.name
        func_name = ""
        lineno = 0
        
        candidate = frame.f_globals.get("__name__", "")
        module_name = candidate
        func_name = frame.f_code.co_name
        lineno = frame.f_lineno
        
        return {
            'caller_module': module_name, 
            'caller_function': func_name, 
            'caller_lineno': lineno
        }
    
    def debug(self, message: str, *args, **kwargs):
        extra = self._get_caller_info()
        self.logger.debug(message, extra=extra, *args, **kwargs)
    
    def error(self, message: str, *args, **kwargs):
        traceback.print_stack()
        extra = self._get_caller_info()
        self.logger.error(message, extra=extra, *args, **kwargs)

    def info(self, message: str, *args, **kwargs):
        extra = self._get_caller_info()
        self.logger.info(message, extra=extra, *args, **kwargs)

    def warning(self, message: str, *args, **kwargs):
        extra = self._get_caller_info()
        self.logger.warning(message, extra=extra, *args, **kwargs)

    def critical(self, message: str, *args, **kwargs):
        extra = self._get_caller_info()
        self.logger.critical(message, extra=extra, *args, **kwargs)


def get_logger(name: str, level: int = logging.DEBUG) -> Logger:
    return Logger(name=name, level=level)


from sqlalchemy import Column, Integer, String, Boolean, Float

from airunner.data.models.base import BaseModel


class WhisperSettings(BaseModel):
    __tablename__ = "whisper_settings"
    id = Column(Integer, primary_key=True, autoincrement=True)
    is_multilingual = Column(Boolean, default=False)
    temperature = Column(Float, default=0.8)
    compression_ratio_threshold = Column(Float, default=1.35)
    logprob_threshold = Column(Float, default=-1.0)
    no_speech_threshold = Column(Float, default=0.2)
    time_precision = Column(Float, default=0.02)
    language = Column(String, default="en")
    task = Column(String, default="transcribe")



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'generator_tab_backup.ui'
##
## Created by: Qt User Interface Compiler version 6.7.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QGridLayout, QSizePolicy, QTabWidget,
    QWidget)

from airunner.gui.widgets.generator_form.generator_form_widget import GeneratorForm

class Ui_generator_tab(object):
    def setupUi(self, generator_tab):
        if not generator_tab.objectName():
            generator_tab.setObjectName(u"generator_tab")
        generator_tab.resize(330, 972)
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(generator_tab.sizePolicy().hasHeightForWidth())
        generator_tab.setSizePolicy(sizePolicy)
        self.gridLayout_2 = QGridLayout(generator_tab)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.generator_tabs = QTabWidget(generator_tab)
        self.generator_tabs.setObjectName(u"generator_tabs")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.generator_tabs.sizePolicy().hasHeightForWidth())
        self.generator_tabs.setSizePolicy(sizePolicy1)
        self.tab_stablediffusion = QWidget()
        self.tab_stablediffusion.setObjectName(u"tab_stablediffusion")
        sizePolicy2 = QSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Preferred)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.tab_stablediffusion.sizePolicy().hasHeightForWidth())
        self.tab_stablediffusion.setSizePolicy(sizePolicy2)
        self.gridLayout = QGridLayout(self.tab_stablediffusion)
        self.gridLayout.setObjectName(u"gridLayout")
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.tab_widget_stablediffusion = QTabWidget(self.tab_stablediffusion)
        self.tab_widget_stablediffusion.setObjectName(u"tab_widget_stablediffusion")
        self.tab_widget_stablediffusion.setTabPosition(QTabWidget.West)
        self.tab_stablediffusion_txt2img = QWidget()
        self.tab_stablediffusion_txt2img.setObjectName(u"tab_stablediffusion_txt2img")
        sizePolicy2.setHeightForWidth(self.tab_stablediffusion_txt2img.sizePolicy().hasHeightForWidth())
        self.tab_stablediffusion_txt2img.setSizePolicy(sizePolicy2)
        self.gridLayout_7 = QGridLayout(self.tab_stablediffusion_txt2img)
        self.gridLayout_7.setObjectName(u"gridLayout_7")
        self.gridLayout_7.setContentsMargins(0, 0, 0, 0)
        self.generator_form_stablediffusion_txt2img = GeneratorForm(self.tab_stablediffusion_txt2img)
        self.generator_form_stablediffusion_txt2img.setObjectName(u"generator_form_stablediffusion_txt2img")

        self.gridLayout_7.addWidget(self.generator_form_stablediffusion_txt2img, 0, 0, 1, 1)

        self.tab_widget_stablediffusion.addTab(self.tab_stablediffusion_txt2img, "")
        self.tab_stablediffusion_outpaint = QWidget()
        self.tab_stablediffusion_outpaint.setObjectName(u"tab_stablediffusion_outpaint")
        sizePolicy2.setHeightForWidth(self.tab_stablediffusion_outpaint.sizePolicy().hasHeightForWidth())
        self.tab_stablediffusion_outpaint.setSizePolicy(sizePolicy2)
        self.gridLayout_10 = QGridLayout(self.tab_stablediffusion_outpaint)
        self.gridLayout_10.setObjectName(u"gridLayout_10")
        self.gridLayout_10.setContentsMargins(0, 0, 0, 0)
        self.generator_form_stablediffusion_outpaint = GeneratorForm(self.tab_stablediffusion_outpaint)
        self.generator_form_stablediffusion_outpaint.setObjectName(u"generator_form_stablediffusion_outpaint")

        self.gridLayout_10.addWidget(self.generator_form_stablediffusion_outpaint, 0, 0, 1, 1)

        self.tab_widget_stablediffusion.addTab(self.tab_stablediffusion_outpaint, "")
        self.tab_stablediffusion_depth2img = QWidget()
        self.tab_stablediffusion_depth2img.setObjectName(u"tab_stablediffusion_depth2img")
        sizePolicy2.setHeightForWidth(self.tab_stablediffusion_depth2img.sizePolicy().hasHeightForWidth())
        self.tab_stablediffusion_depth2img.setSizePolicy(sizePolicy2)
        self.gridLayout_11 = QGridLayout(self.tab_stablediffusion_depth2img)
        self.gridLayout_11.setObjectName(u"gridLayout_11")
        self.gridLayout_11.setContentsMargins(0, 0, 0, 0)
        self.generator_form_stablediffusion_depth2img = GeneratorForm(self.tab_stablediffusion_depth2img)
        self.generator_form_stablediffusion_depth2img.setObjectName(u"generator_form_stablediffusion_depth2img")

        self.gridLayout_11.addWidget(self.generator_form_stablediffusion_depth2img, 0, 0, 1, 1)

        self.tab_widget_stablediffusion.addTab(self.tab_stablediffusion_depth2img, "")
        self.tab_stablediffusion_pix2pix = QWidget()
        self.tab_stablediffusion_pix2pix.setObjectName(u"tab_stablediffusion_pix2pix")
        sizePolicy2.setHeightForWidth(self.tab_stablediffusion_pix2pix.sizePolicy().hasHeightForWidth())
        self.tab_stablediffusion_pix2pix.setSizePolicy(sizePolicy2)
        self.gridLayout_12 = QGridLayout(self.tab_stablediffusion_pix2pix)
        self.gridLayout_12.setObjectName(u"gridLayout_12")
        self.gridLayout_12.setContentsMargins(0, 0, 0, 0)
        self.generator_form_stablediffusion_pix2pix = GeneratorForm(self.tab_stablediffusion_pix2pix)
        self.generator_form_stablediffusion_pix2pix.setObjectName(u"generator_form_stablediffusion_pix2pix")

        self.gridLayout_12.addWidget(self.generator_form_stablediffusion_pix2pix, 0, 0, 1, 1)

        self.tab_widget_stablediffusion.addTab(self.tab_stablediffusion_pix2pix, "")
        self.tab_stablediffusion_upscale = QWidget()
        self.tab_stablediffusion_upscale.setObjectName(u"tab_stablediffusion_upscale")
        sizePolicy2.setHeightForWidth(self.tab_stablediffusion_upscale.sizePolicy().hasHeightForWidth())
        self.tab_stablediffusion_upscale.setSizePolicy(sizePolicy2)
        self.gridLayout_5 = QGridLayout(self.tab_stablediffusion_upscale)
        self.gridLayout_5.setObjectName(u"gridLayout_5")
        self.gridLayout_5.setContentsMargins(0, 0, 0, 0)
        self.generator_form_stablediffusion_upscale = GeneratorForm(self.tab_stablediffusion_upscale)
        self.generator_form_stablediffusion_upscale.setObjectName(u"generator_form_stablediffusion_upscale")

        self.gridLayout_5.addWidget(self.generator_form_stablediffusion_upscale, 0, 0, 1, 1)

        self.tab_widget_stablediffusion.addTab(self.tab_stablediffusion_upscale, "")
        self.tab_stablediffusion_superresolution = QWidget()
        self.tab_stablediffusion_superresolution.setObjectName(u"tab_stablediffusion_superresolution")
        sizePolicy2.setHeightForWidth(self.tab_stablediffusion_superresolution.sizePolicy().hasHeightForWidth())
        self.tab_stablediffusion_superresolution.setSizePolicy(sizePolicy2)
        self.gridLayout_6 = QGridLayout(self.tab_stablediffusion_superresolution)
        self.gridLayout_6.setObjectName(u"gridLayout_6")
        self.gridLayout_6.setContentsMargins(0, 0, 0, 0)
        self.generator_form_stablediffusion_superresolution = GeneratorForm(self.tab_stablediffusion_superresolution)
        self.generator_form_stablediffusion_superresolution.setObjectName(u"generator_form_stablediffusion_superresolution")

        self.gridLayout_6.addWidget(self.generator_form_stablediffusion_superresolution, 0, 0, 1, 1)

        self.tab_widget_stablediffusion.addTab(self.tab_stablediffusion_superresolution, "")
        self.tab_stablediffusion_txt2vid = QWidget()
        self.tab_stablediffusion_txt2vid.setObjectName(u"tab_stablediffusion_txt2vid")
        sizePolicy2.setHeightForWidth(self.tab_stablediffusion_txt2vid.sizePolicy().hasHeightForWidth())
        self.tab_stablediffusion_txt2vid.setSizePolicy(sizePolicy2)
        self.gridLayout_13 = QGridLayout(self.tab_stablediffusion_txt2vid)
        self.gridLayout_13.setObjectName(u"gridLayout_13")
        self.gridLayout_13.setContentsMargins(0, 0, 0, 0)
        self.generator_form_txt2vid = GeneratorForm(self.tab_stablediffusion_txt2vid)
        self.generator_form_txt2vid.setObjectName(u"generator_form_txt2vid")

        self.gridLayout_13.addWidget(self.generator_form_txt2vid, 0, 0, 1, 1)

        self.tab_widget_stablediffusion.addTab(self.tab_stablediffusion_txt2vid, "")

        self.gridLayout.addWidget(self.tab_widget_stablediffusion, 0, 0, 1, 1)

        self.generator_tabs.addTab(self.tab_stablediffusion, "")
        self.tab_kandinsky = QWidget()
        self.tab_kandinsky.setObjectName(u"tab_kandinsky")
        sizePolicy2.setHeightForWidth(self.tab_kandinsky.sizePolicy().hasHeightForWidth())
        self.tab_kandinsky.setSizePolicy(sizePolicy2)
        self.gridLayout_3 = QGridLayout(self.tab_kandinsky)
        self.gridLayout_3.setObjectName(u"gridLayout_3")
        self.gridLayout_3.setContentsMargins(0, 0, 0, 0)
        self.tab_widget_kandinsky = QTabWidget(self.tab_kandinsky)
        self.tab_widget_kandinsky.setObjectName(u"tab_widget_kandinsky")
        self.tab_widget_kandinsky.setTabPosition(QTabWidget.West)
        self.tab_kandinsky_txt2img = QWidget()
        self.tab_kandinsky_txt2img.setObjectName(u"tab_kandinsky_txt2img")
        sizePolicy2.setHeightForWidth(self.tab_kandinsky_txt2img.sizePolicy().hasHeightForWidth())
        self.tab_kandinsky_txt2img.setSizePolicy(sizePolicy2)
        self.gridLayout_8 = QGridLayout(self.tab_kandinsky_txt2img)
        self.gridLayout_8.setObjectName(u"gridLayout_8")
        self.gridLayout_8.setContentsMargins(0, 0, 0, 0)
        self.generator_form_kandinsky_txt2img = GeneratorForm(self.tab_kandinsky_txt2img)
        self.generator_form_kandinsky_txt2img.setObjectName(u"generator_form_kandinsky_txt2img")

        self.gridLayout_8.addWidget(self.generator_form_kandinsky_txt2img, 0, 0, 1, 1)

        self.tab_widget_kandinsky.addTab(self.tab_kandinsky_txt2img, "")
        self.tab_kandinsky_outpaint = QWidget()
        self.tab_kandinsky_outpaint.setObjectName(u"tab_kandinsky_outpaint")
        sizePolicy2.setHeightForWidth(self.tab_kandinsky_outpaint.sizePolicy().hasHeightForWidth())
        self.tab_kandinsky_outpaint.setSizePolicy(sizePolicy2)
        self.gridLayout_14 = QGridLayout(self.tab_kandinsky_outpaint)
        self.gridLayout_14.setObjectName(u"gridLayout_14")
        self.gridLayout_14.setContentsMargins(0, 0, 0, 0)
        self.generator_form_kandinsky_outpaint = GeneratorForm(self.tab_kandinsky_outpaint)
        self.generator_form_kandinsky_outpaint.setObjectName(u"generator_form_kandinsky_outpaint")

        self.gridLayout_14.addWidget(self.generator_form_kandinsky_outpaint, 0, 0, 1, 1)

        self.tab_widget_kandinsky.addTab(self.tab_kandinsky_outpaint, "")

        self.gridLayout_3.addWidget(self.tab_widget_kandinsky, 0, 0, 1, 1)

        self.generator_tabs.addTab(self.tab_kandinsky, "")
        self.tab_shape = QWidget()
        self.tab_shape.setObjectName(u"tab_shape")
        sizePolicy2.setHeightForWidth(self.tab_shape.sizePolicy().hasHeightForWidth())
        self.tab_shape.setSizePolicy(sizePolicy2)
        self.gridLayout_4 = QGridLayout(self.tab_shape)
        self.gridLayout_4.setObjectName(u"gridLayout_4")
        self.gridLayout_4.setContentsMargins(0, 0, 0, 0)
        self.tab_widget_shape = QTabWidget(self.tab_shape)
        self.tab_widget_shape.setObjectName(u"tab_widget_shape")
        self.tab_widget_shape.setTabPosition(QTabWidget.West)
        self.tab_shape_txt2img = QWidget()
        self.tab_shape_txt2img.setObjectName(u"tab_shape_txt2img")
        sizePolicy2.setHeightForWidth(self.tab_shape_txt2img.sizePolicy().hasHeightForWidth())
        self.tab_shape_txt2img.setSizePolicy(sizePolicy2)
        self.gridLayout_9 = QGridLayout(self.tab_shape_txt2img)
        self.gridLayout_9.setObjectName(u"gridLayout_9")
        self.gridLayout_9.setContentsMargins(0, 0, 0, 0)
        self.generator_form_shape_txt2img = GeneratorForm(self.tab_shape_txt2img)
        self.generator_form_shape_txt2img.setObjectName(u"generator_form_shape_txt2img")

        self.gridLayout_9.addWidget(self.generator_form_shape_txt2img, 0, 0, 1, 1)

        self.tab_widget_shape.addTab(self.tab_shape_txt2img, "")

        self.gridLayout_4.addWidget(self.tab_widget_shape, 0, 0, 1, 1)

        self.generator_tabs.addTab(self.tab_shape, "")

        self.gridLayout_2.addWidget(self.generator_tabs, 0, 0, 1, 1)


        self.retranslateUi(generator_tab)
        self.generator_tabs.currentChanged.connect(generator_tab.handle_generator_tab_changed)
        self.tab_widget_stablediffusion.currentChanged.connect(generator_tab.handle_tab_section_changed)

        self.generator_tabs.setCurrentIndex(0)
        self.tab_widget_stablediffusion.setCurrentIndex(0)
        self.tab_widget_kandinsky.setCurrentIndex(1)
        self.tab_widget_shape.setCurrentIndex(0)


        QMetaObject.connectSlotsByName(generator_tab)
    # setupUi

    def retranslateUi(self, generator_tab):
        generator_tab.setWindowTitle(QCoreApplication.translate("generator_tab", u"Form", None))
        self.tab_stablediffusion_txt2img.setProperty("generator_section", QCoreApplication.translate("generator_tab", u"txt2img", None))
        self.tab_stablediffusion_txt2img.setProperty("generator_name", QCoreApplication.translate("generator_tab", u"stablediffusion", None))
        self.generator_form_stablediffusion_txt2img.setProperty("generator_section", QCoreApplication.translate("generator_tab", u"txt2img", None))
        self.generator_form_stablediffusion_txt2img.setProperty("generator_name", QCoreApplication.translate("generator_tab", u"stablediffusion", None))
        self.tab_widget_stablediffusion.setTabText(self.tab_widget_stablediffusion.indexOf(self.tab_stablediffusion_txt2img), QCoreApplication.translate("generator_tab", u"txt2img / img2img", None))
        self.tab_stablediffusion_outpaint.setProperty("generator_section", QCoreApplication.translate("generator_tab", u"outpaint", None))
        self.tab_stablediffusion_outpaint.setProperty("generator_name", QCoreApplication.translate("generator_tab", u"stablediffusion", None))
        self.generator_form_stablediffusion_outpaint.setProperty("generator_section", QCoreApplication.translate("generator_tab", u"outpaint", None))
        self.generator_form_stablediffusion_outpaint.setProperty("generator_name", QCoreApplication.translate("generator_tab", u"stablediffusion", None))
        self.tab_widget_stablediffusion.setTabText(self.tab_widget_stablediffusion.indexOf(self.tab_stablediffusion_outpaint), QCoreApplication.translate("generator_tab", u"inpaint / outapint", None))
        self.tab_stablediffusion_depth2img.setProperty("generator_section", QCoreApplication.translate("generator_tab", u"depth2img", None))
        self.tab_stablediffusion_depth2img.setProperty("generator_name", QCoreApplication.translate("generator_tab", u"stablediffusion", None))
        self.generator_form_stablediffusion_depth2img.setProperty("generator_section", QCoreApplication.translate("generator_tab", u"depth2img", None))
        self.generator_form_stablediffusion_depth2img.setProperty("generator_name", QCoreApplication.translate("generator_tab", u"stablediffusion", None))
        self.tab_widget_stablediffusion.setTabText(self.tab_widget_stablediffusion.indexOf(self.tab_stablediffusion_depth2img), QCoreApplication.translate("generator_tab", u"depth2img", None))
        self.tab_stablediffusion_pix2pix.setProperty("generator_section", QCoreApplication.translate("generator_tab", u"pix2pix", None))
        self.tab_stablediffusion_pix2pix.setProperty("generator_name", QCoreApplication.translate("generator_tab", u"stablediffusion", None))
        self.generator_form_stablediffusion_pix2pix.setProperty("generator_section", QCoreApplication.translate("generator_tab", u"pix2pix", None))
        self.generator_form_stablediffusion_pix2pix.setProperty("generator_name", QCoreApplication.translate("generator_tab", u"stablediffusion", None))
        self.tab_widget_stablediffusion.setTabText(self.tab_widget_stablediffusion.indexOf(self.tab_stablediffusion_pix2pix), QCoreApplication.translate("generator_tab", u"pix2pix", None))
        self.tab_stablediffusion_upscale.setProperty("generator_section", QCoreApplication.translate("generator_tab", u"upscale", None))
        self.tab_stablediffusion_upscale.setProperty("generator_name", QCoreApplication.translate("generator_tab", u"stablediffusion", None))
        self.generator_form_stablediffusion_upscale.setProperty("generator_section", QCoreApplication.translate("generator_tab", u"upscale", None))
        self.generator_form_stablediffusion_upscale.setProperty("generator_name", QCoreApplication.translate("generator_tab", u"stablediffusion", None))
        self.tab_widget_stablediffusion.setTabText(self.tab_widget_stablediffusion.indexOf(self.tab_stablediffusion_upscale), QCoreApplication.translate("generator_tab", u"upscale", None))
        self.tab_stablediffusion_superresolution.setProperty("generator_section", QCoreApplication.translate("generator_tab", u"superresolution", None))
        self.tab_stablediffusion_superresolution.setProperty("generator_name", QCoreApplication.translate("generator_tab", u"stablediffusion", None))
        self.generator_form_stablediffusion_superresolution.setProperty("generator_section", QCoreApplication.translate("generator_tab", u"superresolution", None))
        self.generator_form_stablediffusion_superresolution.setProperty("generator_name", QCoreApplication.translate("generator_tab", u"stablediffusion", None))
        self.tab_widget_stablediffusion.setTabText(self.tab_widget_stablediffusion.indexOf(self.tab_stablediffusion_superresolution), QCoreApplication.translate("generator_tab", u"superresolution", None))
        self.tab_stablediffusion_txt2vid.setProperty("generator_section", QCoreApplication.translate("generator_tab", u"txt2vid", None))
        self.tab_stablediffusion_txt2vid.setProperty("generator_name", QCoreApplication.translate("generator_tab", u"stablediffusion", None))
        self.generator_form_txt2vid.setProperty("generator_section", QCoreApplication.translate("generator_tab", u"txt2vid", None))
        self.generator_form_txt2vid.setProperty("generator_name", QCoreApplication.translate("generator_tab", u"stablediffusion", None))
        self.tab_widget_stablediffusion.setTabText(self.tab_widget_stablediffusion.indexOf(self.tab_stablediffusion_txt2vid), QCoreApplication.translate("generator_tab", u"txt2vid", None))
        self.generator_tabs.setTabText(self.generator_tabs.indexOf(self.tab_stablediffusion), QCoreApplication.translate("generator_tab", u"Stable Diffusion", None))
        self.tab_kandinsky_txt2img.setProperty("generator_section", QCoreApplication.translate("generator_tab", u"txt2img", None))
        self.tab_kandinsky_txt2img.setProperty("generator_name", QCoreApplication.translate("generator_tab", u"kandinsky", None))
        self.generator_form_kandinsky_txt2img.setProperty("generator_section", QCoreApplication.translate("generator_tab", u"txt2img", None))
        self.generator_form_kandinsky_txt2img.setProperty("generator_name", QCoreApplication.translate("generator_tab", u"kandinsky", None))
        self.tab_widget_kandinsky.setTabText(self.tab_widget_kandinsky.indexOf(self.tab_kandinsky_txt2img), QCoreApplication.translate("generator_tab", u"txt2img / img2img", None))
        self.tab_kandinsky_outpaint.setProperty("generator_section", QCoreApplication.translate("generator_tab", u"outpaint", None))
        self.tab_kandinsky_outpaint.setProperty("generator_name", QCoreApplication.translate("generator_tab", u"kandinsky", None))
        self.generator_form_kandinsky_outpaint.setProperty("generator_section", QCoreApplication.translate("generator_tab", u"outpaint", None))
        self.generator_form_kandinsky_outpaint.setProperty("generator_name", QCoreApplication.translate("generator_tab", u"kandinsky", None))
        self.tab_widget_kandinsky.setTabText(self.tab_widget_kandinsky.indexOf(self.tab_kandinsky_outpaint), QCoreApplication.translate("generator_tab", u"inpaint / outapint", None))
        self.generator_tabs.setTabText(self.generator_tabs.indexOf(self.tab_kandinsky), QCoreApplication.translate("generator_tab", u"Kandinsky", None))
        self.tab_shape_txt2img.setProperty("generator_section", QCoreApplication.translate("generator_tab", u"txt2img", None))
        self.tab_shape_txt2img.setProperty("generator_name", QCoreApplication.translate("generator_tab", u"shape", None))
        self.generator_form_shape_txt2img.setProperty("generator_section", QCoreApplication.translate("generator_tab", u"txt2img", None))
        self.generator_form_shape_txt2img.setProperty("generator_name", QCoreApplication.translate("generator_tab", u"shape", None))
        self.tab_widget_shape.setTabText(self.tab_widget_shape.indexOf(self.tab_shape_txt2img), QCoreApplication.translate("generator_tab", u"txt2img / img2img", None))
        self.generator_tabs.setTabText(self.generator_tabs.indexOf(self.tab_shape), QCoreApplication.translate("generator_tab", u"Shap-e", None))
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'open_voice_preferences.ui'
##
## Created by: Qt User Interface Compiler version 6.7.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QComboBox, QGridLayout, QGroupBox,
    QSizePolicy, QSpacerItem, QVBoxLayout, QWidget)

from airunner.gui.widgets.slider.slider_widget import SliderWidget

class Ui_open_voice_preferences(object):
    def setupUi(self, open_voice_preferences):
        if not open_voice_preferences.objectName():
            open_voice_preferences.setObjectName(u"open_voice_preferences")
        open_voice_preferences.resize(400, 300)
        self.verticalLayout = QVBoxLayout(open_voice_preferences)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.groupBox = QGroupBox(open_voice_preferences)
        self.groupBox.setObjectName(u"groupBox")
        self.gridLayout = QGridLayout(self.groupBox)
        self.gridLayout.setObjectName(u"gridLayout")
        self.language_combobox = QComboBox(self.groupBox)
        self.language_combobox.setObjectName(u"language_combobox")

        self.gridLayout.addWidget(self.language_combobox, 0, 0, 1, 1)


        self.verticalLayout.addWidget(self.groupBox)

        self.speed_slider = SliderWidget(open_voice_preferences)
        self.speed_slider.setObjectName(u"speed_slider")
        self.speed_slider.setProperty("slider_minimum", 1)
        self.speed_slider.setProperty("slider_maximum", 100)
        self.speed_slider.setProperty("spinbox_minimum", 0.000000000000000)
        self.speed_slider.setProperty("spinbox_maximum", 1.000000000000000)
        self.speed_slider.setProperty("display_as_float", True)
        self.speed_slider.setProperty("slider_single_step", 0)
        self.speed_slider.setProperty("slider_page_step", 0)
        self.speed_slider.setProperty("spinbox_single_step", 0.010000000000000)
        self.speed_slider.setProperty("spinbox_double_step", 0.100000000000000)

        self.verticalLayout.addWidget(self.speed_slider)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.verticalLayout.addItem(self.verticalSpacer)


        self.retranslateUi(open_voice_preferences)
        self.language_combobox.currentTextChanged.connect(open_voice_preferences.language_changed)

        QMetaObject.connectSlotsByName(open_voice_preferences)
    # setupUi

    def retranslateUi(self, open_voice_preferences):
        self.groupBox.setTitle(QCoreApplication.translate("open_voice_preferences", u"Language", None))
        self.language_combobox.setCurrentText("")
        self.speed_slider.setProperty("settings_property", QCoreApplication.translate("open_voice_preferences", u"openvoice_settings.speed", None))
        self.speed_slider.setProperty("label_text", QCoreApplication.translate("open_voice_preferences", u"Speed", None))
        pass
    # retranslateUi




from typing import List, Optional, Any, Dict
from PIL.Image import Image
from dataclasses import dataclass
from airunner.handlers.stablediffusion.rect import Rect

@dataclass
class ImageResponse:
    """
    Represents the response for an image generation request.

    Attributes:
        images: A list of generated images.
        data: Additional metadata related to the image generation.
        nsfw_content_detected: Flag indicating if NSFW content was detected.
        active_rect: The active rectangular region in the image.
        is_outpaint: Flag indicating if the image is an outpainting.
    """
    images: Optional[List[Image]]
    data: Dict[str, Any]
    nsfw_content_detected: bool
    active_rect: Rect
    is_outpaint: bool

    def to_dict(self) -> Dict:
        """
        Convert the ImageResponse object to a dictionary.

        :return: A dictionary representation of the ImageResponse.
        """
        return {
            "images": self.images,
            "data": self.data,
            "nsfw_content_detected": self.nsfw_content_detected,
            "active_rect": {
                "x": self.active_rect.x,
                "y": self.active_rect.y,
                "width": self.active_rect.width,
                "height": self.active_rect.height,
            },
            "is_outpaint": self.is_outpaint,
        }



from airunner.gui.windows.setup_wizard.model_setup.llm.templates.mistral_license_ui import Ui_mistral_license
from airunner.gui.windows.setup_wizard.user_agreement.agreement_page import AgreementPage


class MistralLicense(AgreementPage):
    class_name_ = Ui_mistral_license





# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'image_generator_preferences.ui'
##
## Created by: Qt User Interface Compiler version 6.7.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QComboBox, QGridLayout, QGroupBox,
    QLabel, QSizePolicy, QSpacerItem, QWidget)

from airunner.gui.widgets.stablediffusion.stable_diffusion_settings_widget import StableDiffusionSettingsWidget

class Ui_image_generator_preferences(object):
    def setupUi(self, image_generator_preferences):
        if not image_generator_preferences.objectName():
            image_generator_preferences.setObjectName(u"image_generator_preferences")
        image_generator_preferences.resize(496, 416)
        self.gridLayout = QGridLayout(image_generator_preferences)
        self.gridLayout.setObjectName(u"gridLayout")
        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.gridLayout.addItem(self.verticalSpacer, 1, 0, 1, 1)

        self.groupBox = QGroupBox(image_generator_preferences)
        self.groupBox.setObjectName(u"groupBox")
        self.gridLayout_2 = QGridLayout(self.groupBox)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.label_4 = QLabel(self.groupBox)
        self.label_4.setObjectName(u"label_4")

        self.gridLayout_2.addWidget(self.label_4, 7, 0, 1, 1)

        self.action = QComboBox(self.groupBox)
        self.action.setObjectName(u"action")

        self.gridLayout_2.addWidget(self.action, 8, 0, 1, 1)

        self.pipeline = QComboBox(self.groupBox)
        self.pipeline.setObjectName(u"pipeline")

        self.gridLayout_2.addWidget(self.pipeline, 4, 0, 1, 1)

        self.label = QLabel(self.groupBox)
        self.label.setObjectName(u"label")

        self.gridLayout_2.addWidget(self.label, 0, 0, 1, 1)

        self.label_2 = QLabel(self.groupBox)
        self.label_2.setObjectName(u"label_2")

        self.gridLayout_2.addWidget(self.label_2, 3, 0, 1, 1)

        self.category = QComboBox(self.groupBox)
        self.category.setObjectName(u"category")

        self.gridLayout_2.addWidget(self.category, 1, 0, 1, 1)

        self.generator_form = StableDiffusionSettingsWidget(self.groupBox)
        self.generator_form.setObjectName(u"generator_form")

        self.gridLayout_2.addWidget(self.generator_form, 9, 0, 1, 1)


        self.gridLayout.addWidget(self.groupBox, 0, 0, 1, 1)


        self.retranslateUi(image_generator_preferences)
        self.category.currentTextChanged.connect(image_generator_preferences.category_changed)
        self.pipeline.currentTextChanged.connect(image_generator_preferences.pipeline_changed)
        self.action.currentTextChanged.connect(image_generator_preferences.action_changed)

        QMetaObject.connectSlotsByName(image_generator_preferences)
    # setupUi

    def retranslateUi(self, image_generator_preferences):
        image_generator_preferences.setWindowTitle(QCoreApplication.translate("image_generator_preferences", u"Form", None))
        self.groupBox.setTitle("")
        self.label_4.setText(QCoreApplication.translate("image_generator_preferences", u"Action", None))
        self.label.setText(QCoreApplication.translate("image_generator_preferences", u"Category", None))
        self.label_2.setText(QCoreApplication.translate("image_generator_preferences", u"Pipeline", None))
    # retranslateUi




from typing import Dict
import torch


def gpu_memory_stats(device: torch.device) -> Dict:
    is_cuda = device.type == "cuda"
    
    stats = {
        "total": 0,
        "allocated": 0,
        "free": 0,
        "device_name": "N/A",
    }
    
    if is_cuda:
        stats["total"] = torch.cuda.get_device_properties(device).total_memory / (1024 ** 3)
        stats["allocated"] = torch.cuda.memory_allocated(device) / (1024 ** 3)
        stats["free"] = stats["total"] - stats["allocated"]
        stats["device_name"] = torch.cuda.get_device_name(device)
    
    return stats


from sqlalchemy import Column, Integer, String

from airunner.data.models.base import BaseModel


class Schedulers(BaseModel):
    __tablename__ = 'schedulers'
    id = Column(Integer, primary_key=True, autoincrement=True)
    name = Column(String, nullable=True)
    display_name = Column(String, nullable=True)



import math


def snap_to_grid(settings, x: int, y: int, use_floor: bool = True):
    # Delay the import to avoid circular dependency
    from airunner.data.models.grid_settings import GridSettings

    cell_size = settings.cell_size
    if settings.snap_to_grid:
        x_is_negative = x < 0
        y_is_negative = y < 0

        if use_floor:
            x = abs(x)
            y = abs(y)
            x = math.floor(x / cell_size) * cell_size
            y = math.floor(y / cell_size) * cell_size
            if x_is_negative:
                x = -x
            if y_is_negative:
                y = -y
        else:
            x = round(x / cell_size) * cell_size
            y = round(y / cell_size) * cell_size

    return x, y



from PySide6.QtGui import QFontDatabase, QFont
from PySide6.QtWidgets import QTextEdit, QApplication, QWidget
from PySide6.QtGui import QFontMetrics
from PySide6.QtCore import Qt, QSize, Slot, QEvent, QTimer
from PySide6.QtCore import Signal

from airunner.enums import SignalCode
from airunner.gui.widgets.base_widget import BaseWidget
from airunner.gui.widgets.llm.templates.message_ui import Ui_message
from airunner.data.models import Conversation
from airunner.data.session_manager import session_scope


class AutoResizingTextEdit(QTextEdit):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.document().contentsChanged.connect(self.sizeChange)


class MessageWidget(BaseWidget):
    widget_class_ = Ui_message
    textChanged = Signal()
    icons = [
        ("copy", "copy_button"),
        ("x-circle", "delete_button"),
    ]

    def __init__(self, *args, **kwargs):
        self.signal_handlers = {
            SignalCode.APPLICATION_SETTINGS_CHANGED_SIGNAL: self.on_application_settings_changed_signal,
            SignalCode.DELETE_MESSAGES_AFTER_ID: self.on_delete_messages_after_id,
        }
        self.name = kwargs.pop("name")
        self.message = kwargs.pop("message")
        self.message_id = kwargs.pop("message_id")
        self.conversation_id = kwargs.pop("conversation_id")
        self.is_bot = kwargs.pop("is_bot")
        super().__init__(*args, **kwargs)
        self._deleted = False
        self.ui.content.setReadOnly(True)
        self.ui.content.insertPlainText(self.message)
        self.ui.content.document().contentsChanged.connect(self.sizeChange)
        self.ui.user_name.setText(f"{self.name}")
        self.font_family = None
        self.font_size = None
        self.set_chat_font()

        if self.is_bot:
            self.ui.message_container.setProperty("class", "alternate")

        self.ui.copy_button.setVisible(False)
        self.ui.delete_button.setVisible(False)
        self.ui.message_container.installEventFilter(self)
        self.set_cursor(Qt.CursorShape.ArrowCursor)

    def on_delete_messages_after_id(self, data):
        message_id = data.get("message_id", None)
        if self.message_id > message_id:
            if not self._deleted:  # Check if the widget has already been deleted
                try:
                    if self.parentWidget():  # Check if the widget still has a parent
                        self.deleteLater()
                except RuntimeError:
                    pass

    def set_cursor(self, cursor_type):
        self.ui.message_container.setCursor(cursor_type)
        for child in self.ui.message_container.findChildren(QWidget):
            child.setCursor(cursor_type)

    def eventFilter(self, obj, event):
        if obj == self.ui.message_container:
            if event.type() == QEvent.Type.Enter:
                self.ui.copy_button.setVisible(True)
                self.ui.delete_button.setVisible(True)
            elif event.type() == QEvent.Type.Leave:
                self.ui.copy_button.setVisible(False)
                self.ui.delete_button.setVisible(False)
        return super().eventFilter(obj, event)

    def on_application_settings_changed_signal(self):
        self.set_chat_font()

    def set_chat_font(self):
        font_setting = self.get_font_setting_by_name("chat")
        font_family = font_setting.font_family
        font_size = font_setting.font_size
        if self.font_family != font_family or self.font_size != font_size:
            self.font_family = font_family
            self.font_size = font_size
            # Check if the font family is available
            available_families = QFontDatabase().families()
            if self.font_family in available_families:
                font = QFont(self.font_family, self.font_size)
            else:
                font = QFont("Ubuntu", self.font_size)  # Fallback to Ubuntu if Arial is not available
            font.setFamilies([
                font.family(),
                "Noto Color Emoji",
            ])
            self.ui.content.setFont(font)

    def set_content_size(self):
        doc_height = self.ui.content.document().size().height()
        doc_width = self.ui.content.document().size().width()
        self.setMinimumHeight(int(doc_height) + 45)
        self.setMinimumWidth(int(doc_width))

    def sizeChange(self):
        self.set_content_size()
        self.textChanged.emit()

    def resizeEvent(self, event):
        self.set_content_size()
        super().resizeEvent(event)

    def sizeHint(self):
        fm = QFontMetrics(self.font())
        h = fm.height() * (self.ui.content.document().lineCount() + 1)
        return QSize(self.width(), h)

    def minimumSizeHint(self):
        return self.sizeHint()

    def update_message(self, text):
        self.message += text

        # strip double spaces from self.message
        self.message = self.message.replace("  ", " ")

        self.ui.content.setPlainText(self.message)

    @Slot()
    def delete(self):
        if not self._deleted:  # Check if the widget has already been deleted
            self._deleted = True
            with session_scope() as session:
                conversation = session.query(Conversation).filter(
                    Conversation.id == self.conversation_id
                ).first()
                messages = conversation.value
                if self.message_id == 0:
                    conversation.value = []
                else:
                    conversation.value = messages[0:self.message_id]
                session.add(conversation)
                session.commit()
            self.emit_signal(SignalCode.DELETE_MESSAGES_AFTER_ID, {
                    "message_id": self.message_id,
                })
            self.setParent(None)
            QTimer.singleShot(0, self.deleteLater)

    @Slot()
    def copy(self):
        clipboard = QApplication.clipboard()
        clipboard.setText(self.message)



LLM_FILE_BOOTSTRAP_DATA = {
    "w4ffl35/Ministral-8B-Instruct-2410-doublequant": {
        "path_settings": "llm_causallm_model_path",
        "files": [
            "config.json",
            "generation_config.json",
            "model-00001-of-00002.safetensors",
            "model-00002-of-00002.safetensors",
            "model.safetensors.index.json",
            "params.json",
            "special_tokens_map.json",
            "tekken.json",
            "tokenizer.json",
            "tokenizer_config.json",
        ]
    },
    "sentence-transformers/sentence-t5-large": {
        "path_settings": "sentence_transformers_path",
        "files": [
            "1_Pooling/config.json",
            "2_Dense/config.json",
            "2_Dense/pytorch_model.bin",
            "config.json",
            "config_sentence_transformers.json",
            "model.safetensors",
            "modules.json",
            "sentence_bert_config.json",
            "special_tokens_map.json",
            "tokenizer.json",
            "tokenizer_config.json",
        ]
    },
    "meta-llama/Meta-Llama-3-8B-Instruct": {
        "path_settings": "llm_causallm_model_path",
        "files": [
            "config.json",
            "generation_config.json",
            "model-00001-of-00004.safetensors",
            "model-00002-of-00004.safetensors",
            "model-00003-of-00004.safetensors",
            "model-00004-of-00004.safetensors",
            "model.safetensors.index.json",
            "special_tokens_map.json",
            "tokenizer.json",
            "tokenizer_config.json",
        ]
    },
    "intfloat/e5-large": {
        "path_settings": "text_embedding",
        "files": [
            "1_Pooling/config.json",
            "config.json",
            "model.safetensors",
            "modules.json",
            "sentence_bert_config.json",
            "special_tokens_map.json",
            "tokenizer.json",
            "tokenizer_config.json",
            "vocab.txt"
        ]
    },
}



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'paths.ui'
##
## Created by: Qt User Interface Compiler version 6.7.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QGridLayout, QPushButton, QSizePolicy,
    QSpacerItem, QWidget)

from airunner.gui.widgets.paths.path_widget import PathWidget

class Ui_paths_form(object):
    def setupUi(self, paths_form):
        if not paths_form.objectName():
            paths_form.setObjectName(u"paths_form")
        paths_form.resize(498, 1113)
        self.gridLayout = QGridLayout(paths_form)
        self.gridLayout.setObjectName(u"gridLayout")
        self.ebook_path_widget = PathWidget(paths_form)
        self.ebook_path_widget.setObjectName(u"ebook_path_widget")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ebook_path_widget.sizePolicy().hasHeightForWidth())
        self.ebook_path_widget.setSizePolicy(sizePolicy)
        self.ebook_path_widget.setMinimumSize(QSize(0, 0))

        self.gridLayout.addWidget(self.ebook_path_widget, 3, 0, 1, 1)

        self.image_path_widget = PathWidget(paths_form)
        self.image_path_widget.setObjectName(u"image_path_widget")
        sizePolicy.setHeightForWidth(self.image_path_widget.sizePolicy().hasHeightForWidth())
        self.image_path_widget.setSizePolicy(sizePolicy)
        self.image_path_widget.setMinimumSize(QSize(0, 0))

        self.gridLayout.addWidget(self.image_path_widget, 2, 0, 1, 1)

        self.llama_index_path_widget = PathWidget(paths_form)
        self.llama_index_path_widget.setObjectName(u"llama_index_path_widget")
        sizePolicy.setHeightForWidth(self.llama_index_path_widget.sizePolicy().hasHeightForWidth())
        self.llama_index_path_widget.setSizePolicy(sizePolicy)
        self.llama_index_path_widget.setMinimumSize(QSize(0, 0))

        self.gridLayout.addWidget(self.llama_index_path_widget, 5, 0, 1, 1)

        self.base_path_widget = PathWidget(paths_form)
        self.base_path_widget.setObjectName(u"base_path_widget")
        sizePolicy.setHeightForWidth(self.base_path_widget.sizePolicy().hasHeightForWidth())
        self.base_path_widget.setSizePolicy(sizePolicy)
        self.base_path_widget.setMinimumSize(QSize(0, 0))

        self.gridLayout.addWidget(self.base_path_widget, 1, 0, 1, 1)

        self.documents_path_widget = PathWidget(paths_form)
        self.documents_path_widget.setObjectName(u"documents_path_widget")
        sizePolicy.setHeightForWidth(self.documents_path_widget.sizePolicy().hasHeightForWidth())
        self.documents_path_widget.setSizePolicy(sizePolicy)
        self.documents_path_widget.setMinimumSize(QSize(0, 0))

        self.gridLayout.addWidget(self.documents_path_widget, 4, 0, 1, 1)

        self.auto_button = QPushButton(paths_form)
        self.auto_button.setObjectName(u"auto_button")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Preferred)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.auto_button.sizePolicy().hasHeightForWidth())
        self.auto_button.setSizePolicy(sizePolicy1)
        self.auto_button.setMinimumSize(QSize(0, 0))

        self.gridLayout.addWidget(self.auto_button, 0, 0, 1, 1)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.gridLayout.addItem(self.verticalSpacer, 6, 0, 1, 1)


        self.retranslateUi(paths_form)
        self.auto_button.clicked.connect(paths_form.action_button_clicked_reset)

        QMetaObject.connectSlotsByName(paths_form)
    # setupUi

    def retranslateUi(self, paths_form):
        paths_form.setWindowTitle(QCoreApplication.translate("paths_form", u"Form", None))
        self.ebook_path_widget.setProperty("title", QCoreApplication.translate("paths_form", u"Ebook Path", None))
        self.ebook_path_widget.setProperty("description", QCoreApplication.translate("paths_form", u"Absolute path to directory containing ebooks (epub)", None))
        self.ebook_path_widget.setProperty("path_name", QCoreApplication.translate("paths_form", u"ebook_path", None))
        self.image_path_widget.setProperty("title", QCoreApplication.translate("paths_form", u"Image Path", None))
        self.image_path_widget.setProperty("description", QCoreApplication.translate("paths_form", u"Absolute path to the directory which will contain generated images", None))
        self.image_path_widget.setProperty("path_name", QCoreApplication.translate("paths_form", u"image_path", None))
        self.llama_index_path_widget.setProperty("title", QCoreApplication.translate("paths_form", u"LLama Index Path", None))
        self.llama_index_path_widget.setProperty("description", QCoreApplication.translate("paths_form", u"Absolute path to directory which will store files for LLama Index", None))
        self.llama_index_path_widget.setProperty("path_name", QCoreApplication.translate("paths_form", u"llama_index_path", None))
        self.base_path_widget.setProperty("title", QCoreApplication.translate("paths_form", u"Base Path", None))
        self.base_path_widget.setProperty("description", QCoreApplication.translate("paths_form", u"Absolute path to the directory which will hold all model directories", None))
        self.base_path_widget.setProperty("path_name", QCoreApplication.translate("paths_form", u"base_path", None))
        self.documents_path_widget.setProperty("title", QCoreApplication.translate("paths_form", u"Documents Path", None))
        self.documents_path_widget.setProperty("description", QCoreApplication.translate("paths_form", u"Absolute path to directory containing documnets (txt, PDF)", None))
        self.documents_path_widget.setProperty("path_name", QCoreApplication.translate("paths_form", u"documents_path", None))
        self.auto_button.setText(QCoreApplication.translate("paths_form", u"Reset Paths to Default Values", None))
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'stt_welcome_screen.ui'
##
## Created by: Qt User Interface Compiler version 6.7.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QFrame, QGridLayout, QLabel,
    QRadioButton, QSizePolicy, QSpacerItem, QWidget)

class Ui_stt_welcome_screen(object):
    def setupUi(self, stt_welcome_screen):
        if not stt_welcome_screen.objectName():
            stt_welcome_screen.setObjectName(u"stt_welcome_screen")
        stt_welcome_screen.resize(403, 223)
        self.gridLayout = QGridLayout(stt_welcome_screen)
        self.gridLayout.setObjectName(u"gridLayout")
        self.label = QLabel(stt_welcome_screen)
        self.label.setObjectName(u"label")
        font = QFont()
        font.setPointSize(11)
        font.setBold(True)
        self.label.setFont(font)

        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)

        self.radioButton = QRadioButton(stt_welcome_screen)
        self.radioButton.setObjectName(u"radioButton")
        self.radioButton.setChecked(True)

        self.gridLayout.addWidget(self.radioButton, 4, 0, 1, 1)

        self.radioButton_2 = QRadioButton(stt_welcome_screen)
        self.radioButton_2.setObjectName(u"radioButton_2")

        self.gridLayout.addWidget(self.radioButton_2, 5, 0, 1, 1)

        self.label_2 = QLabel(stt_welcome_screen)
        self.label_2.setObjectName(u"label_2")
        font1 = QFont()
        font1.setBold(True)
        self.label_2.setFont(font1)

        self.gridLayout.addWidget(self.label_2, 3, 0, 1, 1)

        self.verticalSpacer = QSpacerItem(20, 139, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.gridLayout.addItem(self.verticalSpacer, 6, 0, 1, 1)

        self.line = QFrame(stt_welcome_screen)
        self.line.setObjectName(u"line")
        self.line.setFrameShape(QFrame.Shape.HLine)
        self.line.setFrameShadow(QFrame.Shadow.Sunken)

        self.gridLayout.addWidget(self.line, 2, 0, 1, 1)

        self.label_3 = QLabel(stt_welcome_screen)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setWordWrap(True)

        self.gridLayout.addWidget(self.label_3, 1, 0, 1, 1)


        self.retranslateUi(stt_welcome_screen)
        self.radioButton.toggled.connect(stt_welcome_screen.yes_toggled)
        self.radioButton_2.toggled.connect(stt_welcome_screen.no_toggled)

        QMetaObject.connectSlotsByName(stt_welcome_screen)
    # setupUi

    def retranslateUi(self, stt_welcome_screen):
        stt_welcome_screen.setWindowTitle(QCoreApplication.translate("stt_welcome_screen", u"Form", None))
        self.label.setText(QCoreApplication.translate("stt_welcome_screen", u"Speech-To-Text setup", None))
        self.radioButton.setText(QCoreApplication.translate("stt_welcome_screen", u"Yes", None))
        self.radioButton_2.setText(QCoreApplication.translate("stt_welcome_screen", u"No", None))
        self.label_2.setText(QCoreApplication.translate("stt_welcome_screen", u"Would you like to use the Speech-to-Text feature?", None))
        self.label_3.setText(QCoreApplication.translate("stt_welcome_screen", u"Speech-to-text requires the download of an AI model and access to your microphone.", None))
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'confirmation_page.ui'
##
## Created by: Qt User Interface Compiler version 6.7.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QFrame, QGridLayout,
    QGroupBox, QLabel, QScrollArea, QSizePolicy,
    QSpacerItem, QWidget)

class Ui_confirmation_page(object):
    def setupUi(self, confirmation_page):
        if not confirmation_page.objectName():
            confirmation_page.setObjectName(u"confirmation_page")
        confirmation_page.resize(468, 521)
        self.gridLayout_2 = QGridLayout(confirmation_page)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.scrollArea = QScrollArea(confirmation_page)
        self.scrollArea.setObjectName(u"scrollArea")
        self.scrollArea.setWidgetResizable(True)
        self.scrollAreaWidgetContents = QWidget()
        self.scrollAreaWidgetContents.setObjectName(u"scrollAreaWidgetContents")
        self.scrollAreaWidgetContents.setGeometry(QRect(0, 0, 448, 501))
        self.gridLayout = QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout.setObjectName(u"gridLayout")
        self.download_llm_checkbox = QCheckBox(self.scrollAreaWidgetContents)
        self.download_llm_checkbox.setObjectName(u"download_llm_checkbox")

        self.gridLayout.addWidget(self.download_llm_checkbox, 8, 0, 1, 1)

        self.compile_with_pyinstaller_checkbox = QCheckBox(self.scrollAreaWidgetContents)
        self.compile_with_pyinstaller_checkbox.setObjectName(u"compile_with_pyinstaller_checkbox")

        self.gridLayout.addWidget(self.compile_with_pyinstaller_checkbox, 2, 0, 1, 1)

        self.download_controlnet_checkbox = QCheckBox(self.scrollAreaWidgetContents)
        self.download_controlnet_checkbox.setObjectName(u"download_controlnet_checkbox")

        self.gridLayout.addWidget(self.download_controlnet_checkbox, 7, 0, 1, 1)

        self.line_2 = QFrame(self.scrollAreaWidgetContents)
        self.line_2.setObjectName(u"line_2")
        self.line_2.setFrameShape(QFrame.Shape.HLine)
        self.line_2.setFrameShadow(QFrame.Shadow.Sunken)

        self.gridLayout.addWidget(self.line_2, 1, 0, 1, 1)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.gridLayout.addItem(self.verticalSpacer, 12, 0, 1, 1)

        self.download_sd_checkbox = QCheckBox(self.scrollAreaWidgetContents)
        self.download_sd_checkbox.setObjectName(u"download_sd_checkbox")

        self.gridLayout.addWidget(self.download_sd_checkbox, 6, 0, 1, 1)

        self.required_libraries_groupbox = QGroupBox(self.scrollAreaWidgetContents)
        self.required_libraries_groupbox.setObjectName(u"required_libraries_groupbox")
        self.required_libraries_groupbox.setCheckable(True)
        self.required_libraries_groupbox.setChecked(True)
        self.gridLayout_3 = QGridLayout(self.required_libraries_groupbox)
        self.gridLayout_3.setObjectName(u"gridLayout_3")
        self.required_libraries = QScrollArea(self.required_libraries_groupbox)
        self.required_libraries.setObjectName(u"required_libraries")
        self.required_libraries.setWidgetResizable(True)
        self.scrollAreaWidgetContents_2 = QWidget()
        self.scrollAreaWidgetContents_2.setObjectName(u"scrollAreaWidgetContents_2")
        self.scrollAreaWidgetContents_2.setGeometry(QRect(0, 0, 404, 128))
        self.gridLayout_4 = QGridLayout(self.scrollAreaWidgetContents_2)
        self.gridLayout_4.setObjectName(u"gridLayout_4")
        self.required_libraries.setWidget(self.scrollAreaWidgetContents_2)

        self.gridLayout_3.addWidget(self.required_libraries, 0, 0, 1, 1)


        self.gridLayout.addWidget(self.required_libraries_groupbox, 3, 0, 1, 1)

        self.line = QFrame(self.scrollAreaWidgetContents)
        self.line.setObjectName(u"line")
        self.line.setFrameShape(QFrame.Shape.HLine)
        self.line.setFrameShadow(QFrame.Shadow.Sunken)

        self.gridLayout.addWidget(self.line, 4, 0, 1, 1)

        self.download_ai_runner_checkbox = QCheckBox(self.scrollAreaWidgetContents)
        self.download_ai_runner_checkbox.setObjectName(u"download_ai_runner_checkbox")

        self.gridLayout.addWidget(self.download_ai_runner_checkbox, 5, 0, 1, 1)

        self.download_stt_checkbox = QCheckBox(self.scrollAreaWidgetContents)
        self.download_stt_checkbox.setObjectName(u"download_stt_checkbox")

        self.gridLayout.addWidget(self.download_stt_checkbox, 10, 0, 1, 1)

        self.label = QLabel(self.scrollAreaWidgetContents)
        self.label.setObjectName(u"label")
        font = QFont()
        font.setPointSize(9)
        font.setBold(True)
        self.label.setFont(font)
        self.label.setAlignment(Qt.AlignmentFlag.AlignLeading|Qt.AlignmentFlag.AlignLeft|Qt.AlignmentFlag.AlignVCenter)

        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)

        self.download_tts_checkbox = QCheckBox(self.scrollAreaWidgetContents)
        self.download_tts_checkbox.setObjectName(u"download_tts_checkbox")

        self.gridLayout.addWidget(self.download_tts_checkbox, 9, 0, 1, 1)

        self.scrollArea.setWidget(self.scrollAreaWidgetContents)

        self.gridLayout_2.addWidget(self.scrollArea, 0, 0, 1, 1)


        self.retranslateUi(confirmation_page)
        self.download_ai_runner_checkbox.toggled.connect(confirmation_page.toggle_download_ai_runner)
        self.download_sd_checkbox.toggled.connect(confirmation_page.toggle_download_sd)
        self.download_controlnet_checkbox.toggled.connect(confirmation_page.toggle_download_controlnet)
        self.download_llm_checkbox.toggled.connect(confirmation_page.toggle_download_llm)
        self.download_tts_checkbox.toggled.connect(confirmation_page.toggle_download_tts)
        self.download_stt_checkbox.toggled.connect(confirmation_page.toggle_download_stt)
        self.required_libraries_groupbox.toggled.connect(confirmation_page.toggle_all_required)
        self.compile_with_pyinstaller_checkbox.toggled.connect(confirmation_page.toggle_compile_with_pyinstaller)

        QMetaObject.connectSlotsByName(confirmation_page)
    # setupUi

    def retranslateUi(self, confirmation_page):
        confirmation_page.setWindowTitle(QCoreApplication.translate("confirmation_page", u"Form", None))
        self.download_llm_checkbox.setText(QCoreApplication.translate("confirmation_page", u"Download LLM", None))
        self.compile_with_pyinstaller_checkbox.setText(QCoreApplication.translate("confirmation_page", u"Compile with Pyinstaller", None))
        self.download_controlnet_checkbox.setText(QCoreApplication.translate("confirmation_page", u"Download Controlnet models", None))
        self.download_sd_checkbox.setText(QCoreApplication.translate("confirmation_page", u"Download Stable Diffusion models", None))
        self.required_libraries_groupbox.setTitle(QCoreApplication.translate("confirmation_page", u"Install required libraries (linux)", None))
        self.download_ai_runner_checkbox.setText(QCoreApplication.translate("confirmation_page", u"Download AI Runner", None))
        self.download_stt_checkbox.setText(QCoreApplication.translate("confirmation_page", u"Download Speech to Text models", None))
        self.label.setText(QCoreApplication.translate("confirmation_page", u"Confirm your settings and click next to install AI Runner", None))
        self.download_tts_checkbox.setText(QCoreApplication.translate("confirmation_page", u"Download Text to Speech models", None))
    # retranslateUi




from abc import ABC

from llama_index.core.agent import ReActAgent
from llama_index.core.memory import BaseMemory

ReActAgentMeta = type(ReActAgent)


class ReactAgentEngine(
    ReActAgent, 
    ABC, 
    metaclass=ReActAgentMeta
):
    @property
    def memory(self) -> BaseMemory:
        return self._memory

    @memory.setter
    def memory(self, memory: BaseMemory):
        self._memory = memory



from PySide6.QtWidgets import QVBoxLayout, QSpacerItem, QSizePolicy

from airunner.data.models import LLMGeneratorSettings
from airunner.enums import SignalCode
from airunner.gui.widgets.base_widget import BaseWidget
from airunner.gui.widgets.llm.llm_history_item_widget import LLMHistoryItemWidget
from airunner.gui.widgets.llm.templates.llm_history_widget_ui import Ui_llm_history_widget
from airunner.data.models import Conversation


class LLMHistoryWidget(BaseWidget):
    widget_class_ = Ui_llm_history_widget

    def __init__(self, *args, **kwargs):
        super(LLMHistoryWidget, self).__init__(*args, **kwargs)
        self.spacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

    def showEvent(self, event):
        super(LLMHistoryWidget, self).showEvent(event)
        self.load_conversations()

    def load_conversations(self):
        conversations = Conversation.objects.order_by(
            Conversation.id.desc()
        ).all()        

        layout = self.ui.gridLayout_2

        if layout is None:
            self.logger.error("Layout is None")

        # clear all widgets from the layout
        try:
            layout.removeItem(self.spacer)
        except Exception as e:
            self.logger.error(f"Error removing spacer: {e}")

        if layout:
            for i in reversed(range(layout.count())):
                widget = layout.itemAt(i).widget()
                if widget:
                    widget.setParent(None)

        if layout is None:
            layout = QVBoxLayout(self.ui.scrollAreaWidgetContents)
            self.ui.scrollAreaWidgetContents.setLayout(layout)

        for conversation in conversations:
            # if conversation.title == "":
            #     continue
            llm_history_item_widget = LLMHistoryItemWidget(
                conversation=conversation
            )
            # add to layout
            layout.addWidget(llm_history_item_widget)

        # Add a vertical spacer at the end
        layout.addItem(self.spacer)

        self.ui.scrollAreaWidgetContents.setLayout(layout)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'path.ui'
##
## Created by: Qt User Interface Compiler version 6.7.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QFrame, QGridLayout, QHBoxLayout,
    QLabel, QLineEdit, QPushButton, QSizePolicy,
    QWidget)

class Ui_path_widget(object):
    def setupUi(self, path_widget):
        if not path_widget.objectName():
            path_widget.setObjectName(u"path_widget")
        path_widget.resize(452, 94)
        self.gridLayout = QGridLayout(path_widget)
        self.gridLayout.setObjectName(u"gridLayout")
        self.horizontal_line = QFrame(path_widget)
        self.horizontal_line.setObjectName(u"horizontal_line")
        self.horizontal_line.setFrameShape(QFrame.Shape.HLine)
        self.horizontal_line.setFrameShadow(QFrame.Shadow.Sunken)

        self.gridLayout.addWidget(self.horizontal_line, 4, 0, 1, 1)

        self.title_label = QLabel(path_widget)
        self.title_label.setObjectName(u"title_label")
        font = QFont()
        font.setPointSize(10)
        font.setBold(True)
        self.title_label.setFont(font)

        self.gridLayout.addWidget(self.title_label, 1, 0, 1, 1)

        self.description_label = QLabel(path_widget)
        self.description_label.setObjectName(u"description_label")
        font1 = QFont()
        font1.setPointSize(10)
        self.description_label.setFont(font1)

        self.gridLayout.addWidget(self.description_label, 2, 0, 1, 1)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.path = QLineEdit(path_widget)
        self.path.setObjectName(u"path")
        self.path.setFont(font1)

        self.horizontalLayout.addWidget(self.path)

        self.browse_button = QPushButton(path_widget)
        self.browse_button.setObjectName(u"browse_button")
        self.browse_button.setFont(font1)

        self.horizontalLayout.addWidget(self.browse_button)


        self.gridLayout.addLayout(self.horizontalLayout, 3, 0, 1, 1)


        self.retranslateUi(path_widget)
        self.path.textChanged.connect(path_widget.action_path_changed)
        self.browse_button.clicked.connect(path_widget.action_button_clicked)

        QMetaObject.connectSlotsByName(path_widget)
    # setupUi

    def retranslateUi(self, path_widget):
        path_widget.setWindowTitle(QCoreApplication.translate("path_widget", u"Form", None))
        self.title_label.setText(QCoreApplication.translate("path_widget", u"Title", None))
        self.description_label.setText(QCoreApplication.translate("path_widget", u"Description", None))
        self.browse_button.setText(QCoreApplication.translate("path_widget", u"Browse", None))
    # retranslateUi




from sqlalchemy import Column, Integer, Boolean

from airunner.data.models.base import BaseModel


class MemorySettings(BaseModel):
    __tablename__ = 'memory_settings'
    id = Column(Integer, primary_key=True, autoincrement=True)
    use_last_channels = Column(Boolean, default=True)
    use_attention_slicing = Column(Boolean, default=False)
    use_tf32 = Column(Boolean, default=False)
    use_enable_vae_slicing = Column(Boolean, default=True)
    use_accelerated_transformers = Column(Boolean, default=True)
    use_tiled_vae = Column(Boolean, default=True)
    enable_model_cpu_offload = Column(Boolean, default=False)
    use_enable_sequential_cpu_offload = Column(Boolean, default=False)
    use_cudnn_benchmark = Column(Boolean, default=True)
    use_torch_compile = Column(Boolean, default=False)
    use_tome_sd = Column(Boolean, default=True)
    tome_sd_ratio = Column(Integer, default=600)
    move_unused_model_to_cpu = Column(Boolean, default=False)
    unload_unused_models = Column(Boolean, default=True)
    default_gpu_sd = Column(Integer, default=0)
    default_gpu_llm = Column(Integer, default=0)
    default_gpu_tts = Column(Integer, default=0)
    default_gpu_stt = Column(Integer, default=0)
    prevent_unload_on_llm_image_generation = Column(Boolean, default=False)